@@ -19,34 +19,42 @@
 pub struct NonEmpty<T>(T, Vec<T>);
 
 impl<T> NonEmpty<T> {
-    pub fn new(e: T) -> Self {
+    /// Alias for [`NonEmpty::singleton`].
+    pub const fn new(e: T) -> Self {
         Self::singleton(e)
     }
 
-    pub fn singleton(e: T) -> Self {
+    /// Create a new non-empty list with an initial element.
+    pub const fn singleton(e: T) -> Self {
         NonEmpty(e, Vec::new())
     }
 
+    /// Always returns false.
     pub const fn is_empty(&self) -> bool {
         false
     }
 
+    /// Get the first element. Never fails.
     pub const fn first(&self) -> &T {
         &self.0
     }
 
+    /// Push an element to the end of the list.
     pub fn push(&mut self, e: T) {
         self.1.push(e)
     }
 
+    /// Pop an element from the end of the list.
     pub fn pop(&mut self) -> Option<T> {
         self.1.pop()
     }
 
+    /// Get the length of the list.
     pub fn len(&self) -> usize {
         self.1.len() + 1
     }
 
+    /// Get the last element. Never fails.
     pub fn last(&self) -> &T {
         match self.1.last() {
             None => &self.0,
@@ -54,6 +62,7 @@ impl<T> NonEmpty<T> {
         }
     }
 
+    /// Get the last element mutably.
     pub fn last_mut(&mut self) -> &mut T {
         match self.1.last_mut() {
             None => &mut self.0,
@@ -61,6 +70,7 @@ impl<T> NonEmpty<T> {
         }
     }
 
+    /// Get an element by index.
     pub fn get(&self, index: usize) -> Option<&T> {
         if index == 0 {
             Some(&self.0)
@@ -69,6 +79,7 @@ impl<T> NonEmpty<T> {
         }
     }
 
+    /// Get an element by index, mutably.
     pub fn get_mut(&mut self, index: usize) -> Option<&mut T> {
         if index == 0 {
             Some(&mut self.0)
@@ -77,12 +88,27 @@ impl<T> NonEmpty<T> {
         }
     }
 
+    /// Truncate the list to a certain size. Must be greater than `0`.
     pub fn truncate(&mut self, len: usize) {
         assert!(len >= 1);
         self.1.truncate(len - 1);
     }
 
-    pub fn iter<'a>(&'a self) -> impl Iterator + 'a {
+    /// ```
+    /// use nonempty::NonEmpty;
+    ///
+    /// let mut l = NonEmpty::new(42);
+    /// l.push(36);
+    /// l.push(58);
+    ///
+    /// let mut l_iter = l.iter();
+    ///
+    /// assert_eq!(l_iter.next(), Some(&42));
+    /// assert_eq!(l_iter.next(), Some(&36));
+    /// assert_eq!(l_iter.next(), Some(&58));
+    /// assert_eq!(l_iter.next(), None);
+    /// ```
+    pub fn iter<'a>(&'a self) -> impl Iterator<Item = &T> + 'a {
         std::iter::once(&self.0).chain(self.1.iter())
     }
 }
